@@ -9,6 +9,8 @@ from typing import Any, Dict
 
 from pandas import DataFrame
 
+from freqtrade.constants import Config
+
 
 class IHyperOptLoss(ABC):
     """
@@ -19,11 +21,11 @@ class IHyperOptLoss(ABC):
 
     @staticmethod
     @abstractmethod
-    def hyperopt_loss_function(results: DataFrame, trade_count: int,
+    def hyperopt_loss_function(*, results: DataFrame, trade_count: int,
                                min_date: datetime, max_date: datetime,
-                               config: Dict, processed: Dict[str, DataFrame],
+                               config: Config, processed: Dict[str, DataFrame],
                                backtest_stats: Dict[str, Any],
-                               *args, **kwargs) -> float:
+                               **kwargs) -> float:
         """
         Objective function, returns smaller number for better results
         """
