@@ -1,4 +1,5 @@
 import logging
+from datetime import datetime, timezone
 from pathlib import Path
 from typing import Dict, List, Optional
 
@@ -436,11 +437,11 @@ def create_scatter(
     return None
 
 
-def generate_candlestick_graph(pair: str, data: pd.DataFrame, trades: pd.DataFrame = None, *,
-                               indicators1: List[str] = [],
-                               indicators2: List[str] = [],
-                               plot_config: Dict[str, Dict] = {},
-                               ) -> go.Figure:
+def generate_candlestick_graph(
+        pair: str, data: pd.DataFrame, trades: Optional[pd.DataFrame] = None, *,
+        indicators1: List[str] = [], indicators2: List[str] = [],
+        plot_config: Dict[str, Dict] = {},
+        ) -> go.Figure:
     """
     Generate the graph from the data generated by Backtesting or from DB
     Volume will always be ploted in row2, so Row 1 and 3 are to our disposal for custom indicators
@@ -635,7 +636,7 @@ def load_and_plot_trades(config: Config):
     exchange = ExchangeResolver.load_exchange(config['exchange']['name'], config)
     IStrategy.dp = DataProvider(config, exchange)
     strategy.ft_bot_start()
-    strategy.bot_loop_start()
+    strategy.bot_loop_start(datetime.now(timezone.utc))
     plot_elements = init_plotscript(config, list(exchange.markets), strategy.startup_candle_count)
     timerange = plot_elements['timerange']
     trades = plot_elements['trades']
