@@ -2,10 +2,11 @@
 Price pair list filter
 """
 import logging
-from typing import Any, Dict
+from typing import Any, Dict, Optional
 
 from freqtrade.constants import Config
 from freqtrade.exceptions import OperationalException
+from freqtrade.exchange.types import Ticker
 from freqtrade.plugins.pairlist.IPairList import IPairList
 
 
@@ -64,14 +65,16 @@ class PriceFilter(IPairList):
 
         return f"{self.name} - No price filters configured."
 
-    def _validate_pair(self, pair: str, ticker: Dict[str, Any]) -> bool:
+    def _validate_pair(self, pair: str, ticker: Optional[Ticker]) -> bool:
         """
         Check if if one price-step (pip) is > than a certain barrier.
         :param pair: Pair that's currently validated
-        :param ticker: ticker dict as returned from ccxt.fetch_tickers()
+        :param ticker: ticker dict as returned from ccxt.fetch_ticker
         :return: True if the pair can stay, false if it should be removed
         """
-        if ticker.get('last', None) is None or ticker.get('last') == 0:
+        if ticker and 'last' in ticker and ticker['last'] is not None and ticker.get('last') != 0:
+            price: float = ticker['last']
+        else:
             self.log_once(f"Removed {pair} from whitelist, because "
                           "ticker['last'] is empty (Usually no trade in the last 24h).",
                           logger.info)
@@ -79,8 +82,8 @@ class PriceFilter(IPairList):
 
         # Perform low_price_ratio check.
         if self._low_price_ratio != 0:
-            compare = self._exchange.price_get_one_pip(pair, ticker['last'])
-            changeperc = compare / ticker['last']
+            compare = self._exchange.price_get_one_pip(pair, price)
+            changeperc = compare / price
             if changeperc > self._low_price_ratio:
                 self.log_once(f"Removed {pair} from whitelist, "
                               f"because 1 unit is {changeperc:.3%}", logger.info)
@@ -88,7 +91,6 @@ class PriceFilter(IPairList):
 
         # Perform low_amount check
         if self._max_value != 0:
-            price = ticker['last']
             market = self._exchange.markets[pair]
             limits = market['limits']
             if (limits['amount']['min'] is not None):
@@ -113,14 +115,14 @@ class PriceFilter(IPairList):
 
         # Perform min_price check.
         if self._min_price != 0:
-            if ticker['last'] < self._min_price:
+            if price < self._min_price:
                 self.log_once(f"Removed {pair} from whitelist, "
                               f"because last price < {self._min_price:.8f}", logger.info)
                 return False
 
         # Perform max_price check.
         if self._max_price != 0:
-            if ticker['last'] > self._max_price:
+            if price > self._max_price:
                 self.log_once(f"Removed {pair} from whitelist, "
                               f"because last price > {self._max_price:.8f}", logger.info)
                 return False
