@@ -89,7 +89,8 @@ class IResolver:
             module = importlib.util.module_from_spec(spec)
             try:
                 spec.loader.exec_module(module)  # type: ignore # importlib does not use typehints
-            except (ModuleNotFoundError, SyntaxError, ImportError, NameError) as err:
+            except (AttributeError, ModuleNotFoundError, SyntaxError,
+                    ImportError, NameError) as err:
                 # Catch errors in case a specific module is not installed
                 logger.warning(f"Could not import {module_path} due to '{err}'")
                 if enum_failed:
