@@ -10,7 +10,7 @@ from fastapi.exceptions import HTTPException
 from freqtrade.configuration.config_validation import validate_config_consistency
 from freqtrade.data.btanalysis import get_backtest_resultlist, load_and_merge_backtest_result
 from freqtrade.enums import BacktestState
-from freqtrade.exceptions import DependencyException
+from freqtrade.exceptions import DependencyException, OperationalException
 from freqtrade.misc import deep_merge_dicts
 from freqtrade.rpc.api_server.api_schemas import (BacktestHistoryEntry, BacktestRequest,
                                                   BacktestResponse)
@@ -26,9 +26,10 @@ router = APIRouter()
 
 
 @router.post('/backtest', response_model=BacktestResponse, tags=['webserver', 'backtest'])
-# flake8: noqa: C901
-async def api_start_backtest(bt_settings: BacktestRequest, background_tasks: BackgroundTasks,
-                             config=Depends(get_config), ws_mode=Depends(is_webserver_mode)):
+async def api_start_backtest(  # noqa: C901
+        bt_settings: BacktestRequest, background_tasks: BackgroundTasks,
+        config=Depends(get_config), ws_mode=Depends(is_webserver_mode)):
+    ApiServer._bt['bt_error'] = None
     """Start backtesting if not done so already"""
     if ApiServer._bgtask_running:
         raise RPCException('Bot Background task already running')
@@ -60,30 +61,31 @@ async def api_start_backtest(bt_settings: BacktestRequest, background_tasks: Bac
         asyncio.set_event_loop(asyncio.new_event_loop())
         try:
             # Reload strategy
-            lastconfig = ApiServer._bt_last_config
+            lastconfig = ApiServer._bt['last_config']
             strat = StrategyResolver.load_strategy(btconfig)
             validate_config_consistency(btconfig)
 
             if (
-                not ApiServer._bt
+                not ApiServer._bt['bt']
                 or lastconfig.get('timeframe') != strat.timeframe
                 or lastconfig.get('timeframe_detail') != btconfig.get('timeframe_detail')
                 or lastconfig.get('timerange') != btconfig['timerange']
             ):
                 from freqtrade.optimize.backtesting import Backtesting
-                ApiServer._bt = Backtesting(btconfig)
-                ApiServer._bt.load_bt_data_detail()
+                ApiServer._bt['bt'] = Backtesting(btconfig)
+                ApiServer._bt['bt'].load_bt_data_detail()
             else:
-                ApiServer._bt.config = btconfig
-                ApiServer._bt.init_backtest()
+                ApiServer._bt['bt'].config = btconfig
+                ApiServer._bt['bt'].init_backtest()
             # Only reload data if timeframe changed.
             if (
-                not ApiServer._bt_data
-                or not ApiServer._bt_timerange
+                not ApiServer._bt['data']
+                or not ApiServer._bt['timerange']
                 or lastconfig.get('timeframe') != strat.timeframe
                 or lastconfig.get('timerange') != btconfig['timerange']
             ):
-                ApiServer._bt_data, ApiServer._bt_timerange = ApiServer._bt.load_bt_data()
+                ApiServer._bt['data'], ApiServer._bt['timerange'] = ApiServer._bt[
+                    'bt'].load_bt_data()
 
             lastconfig['timerange'] = btconfig['timerange']
             lastconfig['timeframe'] = strat.timeframe
@@ -91,34 +93,35 @@ async def api_start_backtest(bt_settings: BacktestRequest, background_tasks: Bac
             lastconfig['enable_protections'] = btconfig.get('enable_protections')
             lastconfig['dry_run_wallet'] = btconfig.get('dry_run_wallet')
 
-            ApiServer._bt.enable_protections = btconfig.get('enable_protections', False)
-            ApiServer._bt.strategylist = [strat]
-            ApiServer._bt.results = {}
-            ApiServer._bt.load_prior_backtest()
+            ApiServer._bt['bt'].enable_protections = btconfig.get('enable_protections', False)
+            ApiServer._bt['bt'].strategylist = [strat]
+            ApiServer._bt['bt'].results = {}
+            ApiServer._bt['bt'].load_prior_backtest()
 
-            ApiServer._bt.abort = False
-            if (ApiServer._bt.results and
-                    strat.get_strategy_name() in ApiServer._bt.results['strategy']):
+            ApiServer._bt['bt'].abort = False
+            if (ApiServer._bt['bt'].results and
+                    strat.get_strategy_name() in ApiServer._bt['bt'].results['strategy']):
                 # When previous result hash matches - reuse that result and skip backtesting.
                 logger.info(f'Reusing result of previous backtest for {strat.get_strategy_name()}')
             else:
-                min_date, max_date = ApiServer._bt.backtest_one_strategy(
-                    strat, ApiServer._bt_data, ApiServer._bt_timerange)
+                min_date, max_date = ApiServer._bt['bt'].backtest_one_strategy(
+                    strat, ApiServer._bt['data'], ApiServer._bt['timerange'])
 
-                ApiServer._bt.results = generate_backtest_stats(
-                    ApiServer._bt_data, ApiServer._bt.all_results,
+                ApiServer._bt['bt'].results = generate_backtest_stats(
+                    ApiServer._bt['data'], ApiServer._bt['bt'].all_results,
                     min_date=min_date, max_date=max_date)
 
             if btconfig.get('export', 'none') == 'trades':
                 store_backtest_stats(
-                    btconfig['exportfilename'], ApiServer._bt.results,
+                    btconfig['exportfilename'], ApiServer._bt['bt'].results,
                     datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
                     )
 
             logger.info("Backtest finished.")
 
-        except DependencyException as e:
-            logger.info(f"Backtesting caused an error: {e}")
+        except (Exception, OperationalException, DependencyException) as e:
+            logger.exception(f"Backtesting caused an error: {e}")
+            ApiServer._bt['bt_error'] = str(e)
             pass
         finally:
             ApiServer._bgtask_running = False
@@ -146,13 +149,14 @@ def api_get_backtest(ws_mode=Depends(is_webserver_mode)):
         return {
             "status": "running",
             "running": True,
-            "step": ApiServer._bt.progress.action if ApiServer._bt else str(BacktestState.STARTUP),
-            "progress": ApiServer._bt.progress.progress if ApiServer._bt else 0,
+            "step": (ApiServer._bt['bt'].progress.action if ApiServer._bt['bt']
+                     else str(BacktestState.STARTUP)),
+            "progress": ApiServer._bt['bt'].progress.progress if ApiServer._bt['bt'] else 0,
             "trade_count": len(LocalTrade.trades),
             "status_msg": "Backtest running",
         }
 
-    if not ApiServer._bt:
+    if not ApiServer._bt['bt']:
         return {
             "status": "not_started",
             "running": False,
@@ -160,6 +164,14 @@ def api_get_backtest(ws_mode=Depends(is_webserver_mode)):
             "progress": 0,
             "status_msg": "Backtest not yet executed"
         }
+    if ApiServer._bt['bt_error']:
+        return {
+            "status": "error",
+            "running": False,
+            "step": "",
+            "progress": 0,
+            "status_msg": f"Backtest failed with {ApiServer._bt['bt_error']}"
+        }
 
     return {
         "status": "ended",
@@ -167,7 +179,7 @@ def api_get_backtest(ws_mode=Depends(is_webserver_mode)):
         "status_msg": "Backtest ended",
         "step": "finished",
         "progress": 1,
-        "backtest_result": ApiServer._bt.results,
+        "backtest_result": ApiServer._bt['bt'].results,
     }
 
 
@@ -182,12 +194,12 @@ def api_delete_backtest(ws_mode=Depends(is_webserver_mode)):
             "progress": 0,
             "status_msg": "Backtest running",
         }
-    if ApiServer._bt:
-        ApiServer._bt.cleanup()
-        del ApiServer._bt
-        ApiServer._bt = None
-        del ApiServer._bt_data
-        ApiServer._bt_data = None
+    if ApiServer._bt['bt']:
+        ApiServer._bt['bt'].cleanup()
+        del ApiServer._bt['bt']
+        ApiServer._bt['bt'] = None
+        del ApiServer._bt['data']
+        ApiServer._bt['data'] = None
         logger.info("Backtesting reset")
     return {
         "status": "reset",
@@ -208,7 +220,7 @@ def api_backtest_abort(ws_mode=Depends(is_webserver_mode)):
             "progress": 0,
             "status_msg": "Backtest ended",
         }
-    ApiServer._bt.abort = True
+    ApiServer._bt['bt'].abort = True
     return {
         "status": "stopping",
         "running": False,
@@ -218,14 +230,17 @@ def api_backtest_abort(ws_mode=Depends(is_webserver_mode)):
     }
 
 
-@router.get('/backtest/history', response_model=List[BacktestHistoryEntry], tags=['webserver', 'backtest'])
+@router.get('/backtest/history', response_model=List[BacktestHistoryEntry],
+            tags=['webserver', 'backtest'])
 def api_backtest_history(config=Depends(get_config), ws_mode=Depends(is_webserver_mode)):
     # Get backtest result history, read from metadata files
     return get_backtest_resultlist(config['user_data_dir'] / 'backtest_results')
 
 
-@router.get('/backtest/history/result', response_model=BacktestResponse, tags=['webserver', 'backtest'])
-def api_backtest_history_result(filename: str, strategy: str, config=Depends(get_config), ws_mode=Depends(is_webserver_mode)):
+@router.get('/backtest/history/result', response_model=BacktestResponse,
+            tags=['webserver', 'backtest'])
+def api_backtest_history_result(filename: str, strategy: str, config=Depends(get_config),
+                                ws_mode=Depends(is_webserver_mode)):
     # Get backtest result history, read from metadata files
     fn = config['user_data_dir'] / 'backtest_results' / filename
     results: Dict[str, Any] = {
