@@ -1,9 +1,11 @@
-from typing import Any, Dict, Iterator, Optional
+from typing import Any, AsyncIterator, Dict, Optional
+from uuid import uuid4
 
 from fastapi import Depends
 
 from freqtrade.enums import RunMode
 from freqtrade.persistence import Trade
+from freqtrade.persistence.models import _request_id_ctx_var
 from freqtrade.rpc.rpc import RPC, RPCException
 
 from .webserver import ApiServer
@@ -15,12 +17,19 @@ def get_rpc_optional() -> Optional[RPC]:
     return None
 
 
-def get_rpc() -> Optional[Iterator[RPC]]:
+async def get_rpc() -> Optional[AsyncIterator[RPC]]:
+
     _rpc = get_rpc_optional()
     if _rpc:
+        request_id = str(uuid4())
+        ctx_token = _request_id_ctx_var.set(request_id)
         Trade.rollback()
-        yield _rpc
-        Trade.rollback()
+        try:
+            yield _rpc
+        finally:
+            Trade.session.remove()
+            _request_id_ctx_var.reset(ctx_token)
+
     else:
         raise RPCException('Bot is not in the correct state')
 
