@@ -13,6 +13,7 @@ from freqtrade.exceptions import OperationalException
 from freqtrade.rpc.api_server.uvicorn_threaded import UvicornServer
 from freqtrade.rpc.api_server.ws.message_stream import MessageStream
 from freqtrade.rpc.rpc import RPC, RPCException, RPCHandler
+from freqtrade.rpc.rpc_types import RPCSendMsg
 
 
 logger = logging.getLogger(__name__)
@@ -36,10 +37,13 @@ class ApiServer(RPCHandler):
 
     _rpc: RPC
     # Backtesting type: Backtesting
-    _bt = None
-    _bt_data = None
-    _bt_timerange = None
-    _bt_last_config: Config = {}
+    _bt: Dict[str, Any] = {
+        'bt': None,
+        'data': None,
+        'timerange': None,
+        'last_config': {},
+        'bt_error': None,
+    }
     _has_rpc: bool = False
     _bgtask_running: bool = False
     _config: Config = {}
@@ -105,7 +109,7 @@ class ApiServer(RPCHandler):
         cls._has_rpc = False
         cls._rpc = None
 
-    def send_msg(self, msg: Dict[str, Any]) -> None:
+    def send_msg(self, msg: RPCSendMsg) -> None:
         """
         Publish the message to the message stream
         """
