@@ -2,11 +2,13 @@ from datetime import datetime, timedelta, timezone
 from pathlib import Path
 from unittest.mock import MagicMock, PropertyMock
 
+import ccxt
 import pytest
 
 from freqtrade.enums import CandleType, MarginMode, TradingMode
+from freqtrade.exceptions import RetryableOrderError
 from freqtrade.exchange.exchange import timeframe_to_minutes
-from tests.conftest import get_mock_coro, get_patched_exchange, log_has
+from tests.conftest import EXMS, get_mock_coro, get_patched_exchange, log_has
 from tests.exchange.test_exchange import ccxt_exceptionhandlers
 
 
@@ -46,7 +48,7 @@ def test_get_maintenance_ratio_and_amt_okx(
     default_conf['margin_mode'] = 'isolated'
     default_conf['dry_run'] = False
     mocker.patch.multiple(
-        'freqtrade.exchange.Okx',
+        'freqtrade.exchange.okx.Okx',
         exchange_has=MagicMock(return_value=True),
         load_leverage_tiers=MagicMock(return_value={
             'ETH/USDT:USDT': [
@@ -195,12 +197,12 @@ def test_get_max_pair_stake_amount_okx(default_conf, mocker, leverage_tiers):
     exchange = get_patched_exchange(mocker, default_conf, id="okx")
     exchange._leverage_tiers = leverage_tiers
 
-    assert exchange.get_max_pair_stake_amount('BNB/BUSD', 1.0) == 30000000
-    assert exchange.get_max_pair_stake_amount('BNB/USDT', 1.0) == 50000000
-    assert exchange.get_max_pair_stake_amount('BTC/USDT', 1.0) == 1000000000
-    assert exchange.get_max_pair_stake_amount('BTC/USDT', 1.0, 10.0) == 100000000
+    assert exchange.get_max_pair_stake_amount('BNB/BUSD:BUSD', 1.0) == 30000000
+    assert exchange.get_max_pair_stake_amount('BNB/USDT:USDT', 1.0) == 50000000
+    assert exchange.get_max_pair_stake_amount('BTC/USDT:USDT', 1.0) == 1000000000
+    assert exchange.get_max_pair_stake_amount('BTC/USDT:USDT', 1.0, 10.0) == 100000000
 
-    assert exchange.get_max_pair_stake_amount('TTT/USDT', 1.0) == float('inf')  # Not in tiers
+    assert exchange.get_max_pair_stake_amount('TTT/USDT:USDT', 1.0) == float('inf')  # Not in tiers
 
 
 @pytest.mark.parametrize('mode,side,reduceonly,result', [
@@ -476,3 +478,116 @@ def test_load_leverage_tiers_okx(default_conf, mocker, markets, tmpdir, caplog,
     exchange.load_leverage_tiers()
 
     assert log_has(logmsg, caplog)
+
+
+def test__set_leverage_okx(mocker, default_conf):
+
+    api_mock = MagicMock()
+    api_mock.set_leverage = MagicMock()
+    type(api_mock).has = PropertyMock(return_value={'setLeverage': True})
+    default_conf['dry_run'] = False
+    default_conf['trading_mode'] = TradingMode.FUTURES
+    default_conf['margin_mode'] = MarginMode.ISOLATED
+
+    exchange = get_patched_exchange(mocker, default_conf, api_mock, id="okx")
+    exchange._lev_prep('BTC/USDT:USDT', 3.2, 'buy')
+    assert api_mock.set_leverage.call_count == 1
+    # Leverage is rounded to 3.
+    assert api_mock.set_leverage.call_args_list[0][1]['leverage'] == 3.2
+    assert api_mock.set_leverage.call_args_list[0][1]['symbol'] == 'BTC/USDT:USDT'
+    assert api_mock.set_leverage.call_args_list[0][1]['params'] == {
+        'mgnMode': 'isolated',
+        'posSide': 'net'}
+
+    ccxt_exceptionhandlers(
+        mocker,
+        default_conf,
+        api_mock,
+        "okx",
+        "_lev_prep",
+        "set_leverage",
+        pair="XRP/USDT:USDT",
+        leverage=5.0,
+        side='buy'
+    )
+
+
+@pytest.mark.usefixtures("init_persistence")
+def test_fetch_stoploss_order_okx(default_conf, mocker):
+    default_conf['dry_run'] = False
+    api_mock = MagicMock()
+    api_mock.fetch_order = MagicMock()
+
+    exchange = get_patched_exchange(mocker, default_conf, api_mock, id='okx')
+
+    exchange.fetch_stoploss_order('1234', 'ETH/BTC')
+    assert api_mock.fetch_order.call_count == 1
+    assert api_mock.fetch_order.call_args_list[0][0][0] == '1234'
+    assert api_mock.fetch_order.call_args_list[0][0][1] == 'ETH/BTC'
+    assert api_mock.fetch_order.call_args_list[0][1]['params'] == {'stop': True}
+
+    api_mock.fetch_order = MagicMock(side_effect=ccxt.OrderNotFound)
+    api_mock.fetch_open_orders = MagicMock(return_value=[])
+    api_mock.fetch_closed_orders = MagicMock(return_value=[])
+    api_mock.fetch_canceled_orders = MagicMock(creturn_value=[])
+
+    with pytest.raises(RetryableOrderError):
+        exchange.fetch_stoploss_order('1234', 'ETH/BTC')
+    assert api_mock.fetch_order.call_count == 1
+    assert api_mock.fetch_open_orders.call_count == 1
+    assert api_mock.fetch_closed_orders.call_count == 1
+    assert api_mock.fetch_canceled_orders.call_count == 1
+
+    api_mock.fetch_order.reset_mock()
+    api_mock.fetch_open_orders.reset_mock()
+    api_mock.fetch_closed_orders.reset_mock()
+    api_mock.fetch_canceled_orders.reset_mock()
+
+    api_mock.fetch_closed_orders = MagicMock(return_value=[
+        {
+            'id': '1234',
+            'status': 'closed',
+            'info': {'ordId': '123455'}
+        }
+    ])
+    mocker.patch(f"{EXMS}.fetch_order", MagicMock(return_value={'id': '123455'}))
+    resp = exchange.fetch_stoploss_order('1234', 'ETH/BTC')
+    assert api_mock.fetch_order.call_count == 1
+    assert api_mock.fetch_open_orders.call_count == 1
+    assert api_mock.fetch_closed_orders.call_count == 1
+    assert api_mock.fetch_canceled_orders.call_count == 0
+
+    assert resp['id'] == '1234'
+    assert resp['id_stop'] == '123455'
+    assert resp['type'] == 'stoploss'
+
+    default_conf['dry_run'] = True
+    exchange = get_patched_exchange(mocker, default_conf, api_mock, id='okx')
+    dro_mock = mocker.patch(f"{EXMS}.fetch_dry_run_order", MagicMock(return_value={'id': '123455'}))
+
+    api_mock.fetch_order.reset_mock()
+    api_mock.fetch_open_orders.reset_mock()
+    api_mock.fetch_closed_orders.reset_mock()
+    api_mock.fetch_canceled_orders.reset_mock()
+    resp = exchange.fetch_stoploss_order('1234', 'ETH/BTC')
+
+    assert api_mock.fetch_order.call_count == 0
+    assert api_mock.fetch_open_orders.call_count == 0
+    assert api_mock.fetch_closed_orders.call_count == 0
+    assert api_mock.fetch_canceled_orders.call_count == 0
+    assert dro_mock.call_count == 1
+
+
+@pytest.mark.parametrize('sl1,sl2,sl3,side', [
+    (1501, 1499, 1501, "sell"),
+    (1499, 1501, 1499, "buy")
+])
+def test_stoploss_adjust_okx(mocker, default_conf, sl1, sl2, sl3, side):
+    exchange = get_patched_exchange(mocker, default_conf, id='okx')
+    order = {
+        'type': 'stoploss',
+        'price': 1500,
+        'stopLossPrice': 1500,
+    }
+    assert exchange.stoploss_adjust(sl1, order, side=side)
+    assert not exchange.stoploss_adjust(sl2, order, side=side)
