@@ -5,9 +5,10 @@ from unittest.mock import MagicMock
 import pytest
 
 from freqtrade.data.history import get_timerange
-from freqtrade.enums import ExitType
+from freqtrade.enums import ExitType, TradingMode
 from freqtrade.optimize.backtesting import Backtesting
-from tests.conftest import patch_exchange
+from freqtrade.persistence.trade_model import LocalTrade
+from tests.conftest import EXMS, patch_exchange
 from tests.optimize import (BTContainer, BTrade, _build_backtest_dataframe,
                             _get_frame_time_from_offset, tests_timeframe)
 
@@ -918,17 +919,20 @@ def test_backtest_results(default_conf, fee, mocker, caplog, data: BTContainer)
         default_conf["trailing_stop_positive"] = data.trailing_stop_positive
     default_conf["trailing_stop_positive_offset"] = data.trailing_stop_positive_offset
     default_conf["use_exit_signal"] = data.use_exit_signal
+    default_conf["max_open_trades"] = 10
 
-    mocker.patch("freqtrade.exchange.Exchange.get_fee", return_value=0.0)
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
-    mocker.patch("freqtrade.exchange.Binance.get_max_leverage", return_value=100)
+    mocker.patch(f"{EXMS}.get_fee", return_value=0.0)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=float('inf'))
+    mocker.patch(f"{EXMS}.get_max_leverage", return_value=100)
+    mocker.patch(f"{EXMS}.calculate_funding_fees", return_value=0)
     patch_exchange(mocker)
     frame = _build_backtest_dataframe(data.data)
     backtesting = Backtesting(default_conf)
     # TODO: Should we initialize this properly??
-    backtesting._can_short = True
+    backtesting.trading_mode = TradingMode.MARGIN
     backtesting._set_strategy(backtesting.strategylist[0])
+    backtesting._can_short = True
     backtesting.required_startup = 0
     backtesting.strategy.advise_entry = lambda a, m: frame
     backtesting.strategy.advise_exit = lambda a, m: frame
@@ -950,7 +954,6 @@ def test_backtest_results(default_conf, fee, mocker, caplog, data: BTContainer)
         processed=data_processed,
         start_date=min_date,
         end_date=max_date,
-        max_open_trades=10,
     )
 
     results = result['results']
@@ -964,5 +967,7 @@ def test_backtest_results(default_conf, fee, mocker, caplog, data: BTContainer)
         assert res.open_date == _get_frame_time_from_offset(trade.open_tick)
         assert res.close_date == _get_frame_time_from_offset(trade.close_tick)
         assert res.is_short == trade.is_short
+    assert len(LocalTrade.trades) == len(data.trades)
+    assert len(LocalTrade.trades_open) == 0
     backtesting.cleanup()
     del backtesting
