@@ -19,15 +19,15 @@ from freqtrade.data.btanalysis import BT_DATA_COLUMNS, evaluate_result_multi
 from freqtrade.data.converter import clean_ohlcv_dataframe
 from freqtrade.data.dataprovider import DataProvider
 from freqtrade.data.history import get_timerange
-from freqtrade.enums import ExitType, RunMode
+from freqtrade.enums import CandleType, ExitType, RunMode
 from freqtrade.exceptions import DependencyException, OperationalException
 from freqtrade.exchange.exchange import timeframe_to_next_date
 from freqtrade.optimize.backtest_caching import get_strategy_run_id
 from freqtrade.optimize.backtesting import Backtesting
-from freqtrade.persistence import LocalTrade
+from freqtrade.persistence import LocalTrade, Trade
 from freqtrade.resolvers import StrategyResolver
-from tests.conftest import (CURRENT_TEST_STRATEGY, get_args, log_has, log_has_re, patch_exchange,
-                            patched_configuration_load_config_file)
+from tests.conftest import (CURRENT_TEST_STRATEGY, EXMS, get_args, log_has, log_has_re,
+                            patch_exchange, patched_configuration_load_config_file)
 
 
 ORDER_TYPES = [
@@ -96,7 +96,6 @@ def _make_backtest_conf(mocker, datadir, conf=None, pair='UNITTEST/BTC'):
         'processed': processed,
         'start_date': min_date,
         'end_date': max_date,
-        'max_open_trades': 10,
     }
 
 
@@ -246,7 +245,7 @@ def test_setup_optimize_configuration_stake_amount(mocker, default_conf, caplog)
 
 def test_start(mocker, fee, default_conf, caplog) -> None:
     start_mock = MagicMock()
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
+    mocker.patch(f'{EXMS}.get_fee', fee)
     patch_exchange(mocker)
     mocker.patch('freqtrade.optimize.backtesting.Backtesting.start', start_mock)
     patched_configuration_load_config_file(mocker, default_conf)
@@ -270,7 +269,7 @@ def test_backtesting_init(mocker, default_conf, order_types) -> None:
     """
     default_conf["order_types"] = order_types
     patch_exchange(mocker)
-    get_fee = mocker.patch('freqtrade.exchange.Exchange.get_fee', MagicMock(return_value=0.5))
+    get_fee = mocker.patch(f'{EXMS}.get_fee', MagicMock(return_value=0.5))
     backtesting = Backtesting(default_conf)
     backtesting._set_strategy(backtesting.strategylist[0])
     assert backtesting.config == default_conf
@@ -291,7 +290,7 @@ def test_backtesting_init_no_timeframe(mocker, default_conf, caplog) -> None:
     default_conf['strategy_list'] = [CURRENT_TEST_STRATEGY,
                                      'HyperoptableStrategy']
 
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', MagicMock(return_value=0.5))
+    mocker.patch(f'{EXMS}.get_fee', MagicMock(return_value=0.5))
     with pytest.raises(OperationalException,
                        match=r"Timeframe needs to be set in either configuration"):
         Backtesting(default_conf)
@@ -301,7 +300,7 @@ def test_data_with_fee(default_conf, mocker) -> None:
     patch_exchange(mocker)
     default_conf['fee'] = 0.1234
 
-    fee_mock = mocker.patch('freqtrade.exchange.Exchange.get_fee', MagicMock(return_value=0.5))
+    fee_mock = mocker.patch(f'{EXMS}.get_fee', MagicMock(return_value=0.5))
     backtesting = Backtesting(default_conf)
     backtesting._set_strategy(backtesting.strategylist[0])
     assert backtesting.fee == 0.1234
@@ -345,7 +344,7 @@ def test_backtest_abort(default_conf, mocker, testdatadir) -> None:
     assert backtesting.progress.progress == 0
 
 
-def test_backtesting_start(default_conf, mocker, testdatadir, caplog) -> None:
+def test_backtesting_start(default_conf, mocker, caplog) -> None:
     def get_timerange(input1):
         return Arrow(2017, 11, 14, 21, 17), Arrow(2017, 11, 14, 22, 59)
 
@@ -360,7 +359,6 @@ def test_backtesting_start(default_conf, mocker, testdatadir, caplog) -> None:
                  PropertyMock(return_value=['UNITTEST/BTC']))
 
     default_conf['timeframe'] = '1m'
-    default_conf['datadir'] = testdatadir
     default_conf['export'] = 'signals'
     default_conf['exportfilename'] = 'export.txt'
     default_conf['timerange'] = '-1510694220'
@@ -369,6 +367,7 @@ def test_backtesting_start(default_conf, mocker, testdatadir, caplog) -> None:
     backtesting = Backtesting(default_conf)
     backtesting._set_strategy(backtesting.strategylist[0])
     backtesting.strategy.bot_loop_start = MagicMock()
+    backtesting.strategy.bot_start = MagicMock()
     backtesting.start()
     # check the logs, that will contain the backtest result
     exists = [
@@ -378,7 +377,8 @@ def test_backtesting_start(default_conf, mocker, testdatadir, caplog) -> None:
     for line in exists:
         assert log_has(line, caplog)
     assert backtesting.strategy.dp._pairlists is not None
-    assert backtesting.strategy.bot_loop_start.call_count == 1
+    assert backtesting.strategy.bot_start.call_count == 1
+    assert backtesting.strategy.bot_loop_start.call_count == 0
     assert sbs.call_count == 1
     assert sbc.call_count == 1
 
@@ -396,7 +396,6 @@ def test_backtesting_start_no_data(default_conf, mocker, caplog, testdatadir) ->
                  PropertyMock(return_value=['UNITTEST/BTC']))
 
     default_conf['timeframe'] = "1m"
-    default_conf['datadir'] = testdatadir
     default_conf['export'] = 'none'
     default_conf['timerange'] = '20180101-20180102'
 
@@ -407,7 +406,7 @@ def test_backtesting_start_no_data(default_conf, mocker, caplog, testdatadir) ->
 
 
 def test_backtesting_no_pair_left(default_conf, mocker, caplog, testdatadir) -> None:
-    mocker.patch('freqtrade.exchange.Exchange.exchange_has', MagicMock(return_value=True))
+    mocker.patch(f'{EXMS}.exchange_has', MagicMock(return_value=True))
     mocker.patch('freqtrade.data.history.history_utils.load_pair_history',
                  MagicMock(return_value=pd.DataFrame()))
     mocker.patch('freqtrade.data.history.get_timerange', get_timerange)
@@ -417,7 +416,6 @@ def test_backtesting_no_pair_left(default_conf, mocker, caplog, testdatadir) ->
                  PropertyMock(return_value=[]))
 
     default_conf['timeframe'] = "1m"
-    default_conf['datadir'] = testdatadir
     default_conf['export'] = 'none'
     default_conf['timerange'] = '20180101-20180102'
 
@@ -440,9 +438,9 @@ def test_backtesting_no_pair_left(default_conf, mocker, caplog, testdatadir) ->
 
 
 def test_backtesting_pairlist_list(default_conf, mocker, caplog, testdatadir, tickers) -> None:
-    mocker.patch('freqtrade.exchange.Exchange.exchange_has', MagicMock(return_value=True))
-    mocker.patch('freqtrade.exchange.Exchange.get_tickers', tickers)
-    mocker.patch('freqtrade.exchange.Exchange.price_to_precision', lambda s, x, y: y)
+    mocker.patch(f'{EXMS}.exchange_has', MagicMock(return_value=True))
+    mocker.patch(f'{EXMS}.get_tickers', tickers)
+    mocker.patch(f'{EXMS}.price_to_precision', lambda s, x, y: y)
     mocker.patch('freqtrade.data.history.get_timerange', get_timerange)
     patch_exchange(mocker)
     mocker.patch('freqtrade.optimize.backtesting.Backtesting.backtest')
@@ -451,7 +449,6 @@ def test_backtesting_pairlist_list(default_conf, mocker, caplog, testdatadir, ti
     mocker.patch('freqtrade.plugins.pairlistmanager.PairListManager.refresh_pairlist')
 
     default_conf['ticker_interval'] = "1m"
-    default_conf['datadir'] = testdatadir
     default_conf['export'] = 'none'
     # Use stoploss from strategy
     del default_conf['stoploss']
@@ -479,9 +476,9 @@ def test_backtesting_pairlist_list(default_conf, mocker, caplog, testdatadir, ti
 
 def test_backtest__enter_trade(default_conf, fee, mocker) -> None:
     default_conf['use_exit_signal'] = False
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
+    mocker.patch(f'{EXMS}.get_fee', fee)
+    mocker.patch(f'{EXMS}.get_min_pair_stake_amount', return_value=0.00001)
+    mocker.patch(f'{EXMS}.get_max_pair_stake_amount', return_value=float('inf'))
     patch_exchange(mocker)
     default_conf['stake_amount'] = 'unlimited'
     default_conf['max_open_trades'] = 2
@@ -530,7 +527,7 @@ def test_backtest__enter_trade(default_conf, fee, mocker) -> None:
     assert trade.stake_amount == 495
     assert trade.is_short is True
 
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=300.0)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=300.0)
     trade = backtesting._enter_trade(pair, row=row, direction='long')
     assert trade
     assert trade.stake_amount == 300.0
@@ -538,10 +535,10 @@ def test_backtest__enter_trade(default_conf, fee, mocker) -> None:
 
 def test_backtest__enter_trade_futures(default_conf_usdt, fee, mocker) -> None:
     default_conf_usdt['use_exit_signal'] = False
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
-    mocker.patch("freqtrade.exchange.Exchange.get_max_leverage", return_value=100)
+    mocker.patch(f'{EXMS}.get_fee', fee)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=float('inf'))
+    mocker.patch(f"{EXMS}.get_max_leverage", return_value=100)
     mocker.patch("freqtrade.optimize.backtesting.price_to_precision", lambda p, *args: p)
     patch_exchange(mocker)
     default_conf_usdt['stake_amount'] = 300
@@ -569,7 +566,7 @@ def test_backtest__enter_trade_futures(default_conf_usdt, fee, mocker) -> None:
     ]
 
     backtesting.strategy.leverage = MagicMock(return_value=5.0)
-    mocker.patch("freqtrade.exchange.Exchange.get_maintenance_ratio_and_amt",
+    mocker.patch(f"{EXMS}.get_maintenance_ratio_and_amt",
                  return_value=(0.01, 0.01))
 
     # leverage = 5
@@ -606,7 +603,7 @@ def test_backtest__enter_trade_futures(default_conf_usdt, fee, mocker) -> None:
     assert pytest.approx(trade.liquidation_price) == 0.11787191
 
     # Stake-amount too high!
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=600.0)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=600.0)
 
     trade = backtesting._enter_trade(pair, row=row, direction='long')
     assert trade is None
@@ -619,11 +616,11 @@ def test_backtest__enter_trade_futures(default_conf_usdt, fee, mocker) -> None:
     assert trade is None
 
 
-def test_backtest__get_sell_trade_entry(default_conf, fee, mocker) -> None:
+def test_backtest__check_trade_exit(default_conf, fee, mocker) -> None:
     default_conf['use_exit_signal'] = False
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
+    mocker.patch(f'{EXMS}.get_fee', fee)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=float('inf'))
     patch_exchange(mocker)
     default_conf['timeframe_detail'] = '1m'
     default_conf['max_open_trades'] = 2
@@ -665,7 +662,7 @@ def test_backtest__get_sell_trade_entry(default_conf, fee, mocker) -> None:
     ]
 
     # No data available.
-    res = backtesting._get_exit_trade_entry(trade, row_sell, True)
+    res = backtesting._check_trade_exit(trade, row_sell)
     assert res is not None
     assert res.exit_reason == ExitType.ROI.value
     assert res.close_date_utc == datetime(2020, 1, 1, 5, 0, tzinfo=timezone.utc)
@@ -678,15 +675,17 @@ def test_backtest__get_sell_trade_entry(default_conf, fee, mocker) -> None:
         [], columns=['date', 'open', 'high', 'low', 'close', 'enter_long', 'exit_long',
                      'enter_short', 'exit_short', 'long_tag', 'short_tag', 'exit_tag'])
 
-    res = backtesting._get_exit_trade_entry(trade, row, True)
+    res = backtesting._check_trade_exit(trade, row)
     assert res is None
 
 
 def test_backtest_one(default_conf, fee, mocker, testdatadir) -> None:
     default_conf['use_exit_signal'] = False
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
+    default_conf['max_open_trades'] = 10
+
+    mocker.patch(f'{EXMS}.get_fee', fee)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=float('inf'))
     patch_exchange(mocker)
     backtesting = Backtesting(default_conf)
     backtesting._set_strategy(backtesting.strategylist[0])
@@ -701,7 +700,6 @@ def test_backtest_one(default_conf, fee, mocker, testdatadir) -> None:
         processed=deepcopy(processed),
         start_date=min_date,
         end_date=max_date,
-        max_open_trades=10,
     )
     results = result['results']
     assert not results.empty
@@ -770,9 +768,9 @@ def test_backtest_one(default_conf, fee, mocker, testdatadir) -> None:
 @pytest.mark.parametrize('use_detail', [True, False])
 def test_backtest_one_detail(default_conf_usdt, fee, mocker, testdatadir, use_detail) -> None:
     default_conf_usdt['use_exit_signal'] = False
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
+    mocker.patch(f'{EXMS}.get_fee', fee)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=float('inf'))
     if use_detail:
         default_conf_usdt['timeframe_detail'] = '1m'
     patch_exchange(mocker)
@@ -785,6 +783,8 @@ def test_backtest_one_detail(default_conf_usdt, fee, mocker, testdatadir, use_de
     def custom_entry_price(proposed_rate, **kwargs):
         return proposed_rate * 0.997
 
+    default_conf_usdt['max_open_trades'] = 10
+
     backtesting = Backtesting(default_conf_usdt)
     backtesting._set_strategy(backtesting.strategylist[0])
     backtesting.strategy.populate_entry_trend = advise_entry
@@ -792,10 +792,10 @@ def test_backtest_one_detail(default_conf_usdt, fee, mocker, testdatadir, use_de
     pair = 'XRP/ETH'
     # Pick a timerange adapted to the pair we use to test
     timerange = TimeRange.parse_timerange('20191010-20191013')
-    data = history.load_data(datadir=testdatadir, timeframe='5m', pairs=['XRP/ETH'],
+    data = history.load_data(datadir=testdatadir, timeframe='5m', pairs=[pair],
                              timerange=timerange)
     if use_detail:
-        data_1m = history.load_data(datadir=testdatadir, timeframe='1m', pairs=['XRP/ETH'],
+        data_1m = history.load_data(datadir=testdatadir, timeframe='1m', pairs=[pair],
                                     timerange=timerange)
         backtesting.detail_data = data_1m
     processed = backtesting.strategy.advise_all_indicators(data)
@@ -805,7 +805,6 @@ def test_backtest_one_detail(default_conf_usdt, fee, mocker, testdatadir, use_de
         processed=deepcopy(processed),
         start_date=min_date,
         end_date=max_date,
-        max_open_trades=10,
     )
     results = result['results']
     assert not results.empty
@@ -849,16 +848,175 @@ def test_backtest_one_detail(default_conf_usdt, fee, mocker, testdatadir, use_de
     assert late_entry > 0
 
 
+@pytest.mark.parametrize('use_detail', [True, False])
+def test_backtest_one_detail_futures(
+        default_conf_usdt, fee, mocker, testdatadir, use_detail) -> None:
+    default_conf_usdt['use_exit_signal'] = False
+    default_conf_usdt['trading_mode'] = 'futures'
+    default_conf_usdt['margin_mode'] = 'isolated'
+    default_conf_usdt['candle_type_def'] = CandleType.FUTURES
+
+    mocker.patch(f'{EXMS}.get_fee', fee)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=float('inf'))
+    mocker.patch('freqtrade.plugins.pairlistmanager.PairListManager.whitelist',
+                 PropertyMock(return_value=['XRP/USDT:USDT']))
+    mocker.patch(f"{EXMS}.get_maintenance_ratio_and_amt",
+                 return_value=(0.01, 0.01))
+    default_conf_usdt['timeframe'] = '1h'
+    if use_detail:
+        default_conf_usdt['timeframe_detail'] = '5m'
+    patch_exchange(mocker)
+
+    def advise_entry(df, *args, **kwargs):
+        # Mock function to force several entries
+        df.loc[(df['rsi'] < 40), 'enter_long'] = 1
+        return df
+
+    def custom_entry_price(proposed_rate, **kwargs):
+        return proposed_rate * 0.997
+
+    default_conf_usdt['max_open_trades'] = 10
+
+    backtesting = Backtesting(default_conf_usdt)
+    backtesting._set_strategy(backtesting.strategylist[0])
+    backtesting.strategy.populate_entry_trend = advise_entry
+    backtesting.strategy.custom_entry_price = custom_entry_price
+    pair = 'XRP/USDT:USDT'
+    # Pick a timerange adapted to the pair we use to test
+    timerange = TimeRange.parse_timerange('20211117-20211119')
+    data = history.load_data(datadir=Path(testdatadir), timeframe='1h', pairs=[pair],
+                             timerange=timerange, candle_type=CandleType.FUTURES)
+    backtesting.load_bt_data_detail()
+    processed = backtesting.strategy.advise_all_indicators(data)
+    min_date, max_date = get_timerange(processed)
+
+    result = backtesting.backtest(
+        processed=deepcopy(processed),
+        start_date=min_date,
+        end_date=max_date,
+    )
+    results = result['results']
+    assert not results.empty
+    # Timeout settings from default_conf = entry: 10, exit: 30
+    assert len(results) == (5 if use_detail else 2)
+
+    assert 'orders' in results.columns
+    data_pair = processed[pair]
+
+    data_1m_pair = backtesting.detail_data[pair] if use_detail else pd.DataFrame()
+    late_entry = 0
+    for _, t in results.iterrows():
+        assert len(t['orders']) == 2
+
+        entryo = t['orders'][0]
+        entry_ts = datetime.fromtimestamp(entryo['order_filled_timestamp'] // 1000, tz=timezone.utc)
+        if entry_ts > t['open_date']:
+            late_entry += 1
+
+        # Get "entry fill" candle
+        ln = (data_1m_pair.loc[data_1m_pair["date"] == entry_ts]
+              if use_detail else data_pair.loc[data_pair["date"] == entry_ts])
+        # Check open trade rate aligns to open rate
+        assert not ln.empty
+
+        assert round(ln.iloc[0]["low"], 6) <= round(
+                t["open_rate"], 6) <= round(ln.iloc[0]["high"], 6)
+        # check close trade rate aligns to close rate or is between high and low
+        ln1 = data_pair.loc[data_pair["date"] == t["close_date"]]
+        if use_detail:
+            ln1_1m = data_1m_pair.loc[data_1m_pair["date"] == t["close_date"]]
+            assert not ln1.empty or not ln1_1m.empty
+        else:
+            assert not ln1.empty
+        ln2 = ln1_1m if ln1.empty else ln1
+
+        assert (round(ln2.iloc[0]["low"], 6) <= round(
+                t["close_rate"], 6) <= round(ln2.iloc[0]["high"], 6))
+    assert -0.0181 < Trade.trades[1].funding_fees < -0.01
+    # assert late_entry > 0
+
+
+@pytest.mark.parametrize('use_detail', [True, False])
+def test_backtest_one_detail_futures_funding_fees(
+        default_conf_usdt, fee, mocker, testdatadir, use_detail) -> None:
+    default_conf_usdt['use_exit_signal'] = False
+    default_conf_usdt['trading_mode'] = 'futures'
+    default_conf_usdt['margin_mode'] = 'isolated'
+    default_conf_usdt['candle_type_def'] = CandleType.FUTURES
+    default_conf_usdt['minimal_roi'] = {'0': 1}
+    default_conf_usdt['dry_run_wallet'] = 100000
+
+    mocker.patch(f'{EXMS}.get_fee', fee)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=float('inf'))
+    mocker.patch('freqtrade.plugins.pairlistmanager.PairListManager.whitelist',
+                 PropertyMock(return_value=['XRP/USDT:USDT']))
+    mocker.patch(f"{EXMS}.get_maintenance_ratio_and_amt",
+                 return_value=(0.01, 0.01))
+    default_conf_usdt['timeframe'] = '1h'
+    if use_detail:
+        default_conf_usdt['timeframe_detail'] = '5m'
+    patch_exchange(mocker)
+
+    def advise_entry(df, *args, **kwargs):
+        # Mock function to force several entries
+        df.loc[:, 'enter_long'] = 1
+        return df
+
+    def adjust_trade_position(trade, current_time, **kwargs):
+        if current_time > datetime(2021, 11, 18, 2, 0, 0, tzinfo=timezone.utc):
+            return None
+        return default_conf_usdt['stake_amount']
+
+    default_conf_usdt['max_open_trades'] = 1
+
+    backtesting = Backtesting(default_conf_usdt)
+    backtesting._set_strategy(backtesting.strategylist[0])
+    backtesting.strategy.populate_entry_trend = advise_entry
+    backtesting.strategy.adjust_trade_position = adjust_trade_position
+    backtesting.strategy.leverage = lambda **kwargs: 1
+    backtesting.strategy.position_adjustment_enable = True
+    pair = 'XRP/USDT:USDT'
+    # Pick a timerange adapted to the pair we use to test
+    timerange = TimeRange.parse_timerange('20211117-20211119')
+    data = history.load_data(datadir=Path(testdatadir), timeframe='1h', pairs=[pair],
+                             timerange=timerange, candle_type=CandleType.FUTURES)
+    backtesting.load_bt_data_detail()
+    processed = backtesting.strategy.advise_all_indicators(data)
+    min_date, max_date = get_timerange(processed)
+
+    result = backtesting.backtest(
+        processed=deepcopy(processed),
+        start_date=min_date,
+        end_date=max_date,
+    )
+    results = result['results']
+    assert not results.empty
+    # Only one result - as we're not selling.
+    assert len(results) == 1
+
+    assert 'orders' in results.columns
+
+    for t in Trade.trades:
+        # At least 4 adjustment orders
+        assert t.nr_of_successful_entries >= 6
+        # Funding fees will vary depending on the number of adjustment orders
+        # That number is a lot higher with detail data.
+        assert -20 < t.funding_fees < -0.1
+
+
 def test_backtest_timedout_entry_orders(default_conf, fee, mocker, testdatadir) -> None:
     # This strategy intentionally places unfillable orders.
     default_conf['strategy'] = 'StrategyTestV3CustomEntryPrice'
     default_conf['startup_candle_count'] = 0
     # Cancel unfilled order after 4 minutes on 5m timeframe.
     default_conf["unfilledtimeout"] = {"entry": 4}
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
+    mocker.patch(f'{EXMS}.get_fee', fee)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=float('inf'))
     patch_exchange(mocker)
+    default_conf['max_open_trades'] = 1
     backtesting = Backtesting(default_conf)
     backtesting._set_strategy(backtesting.strategylist[0])
     # Testing dataframe contains 11 candles. Expecting 10 timed out orders.
@@ -871,7 +1029,6 @@ def test_backtest_timedout_entry_orders(default_conf, fee, mocker, testdatadir)
         processed=deepcopy(data),
         start_date=min_date,
         end_date=max_date,
-        max_open_trades=1,
     )
 
     assert result['timedout_entry_orders'] == 10
@@ -879,9 +1036,10 @@ def test_backtest_timedout_entry_orders(default_conf, fee, mocker, testdatadir)
 
 def test_backtest_1min_timeframe(default_conf, fee, mocker, testdatadir) -> None:
     default_conf['use_exit_signal'] = False
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
+    default_conf['max_open_trades'] = 1
+    mocker.patch(f'{EXMS}.get_fee', fee)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=float('inf'))
     patch_exchange(mocker)
     backtesting = Backtesting(default_conf)
     backtesting._set_strategy(backtesting.strategylist[0])
@@ -896,7 +1054,6 @@ def test_backtest_1min_timeframe(default_conf, fee, mocker, testdatadir) -> None
         processed=processed,
         start_date=min_date,
         end_date=max_date,
-        max_open_trades=1,
     )
     assert not results['results'].empty
     assert len(results['results']) == 1
@@ -904,9 +1061,11 @@ def test_backtest_1min_timeframe(default_conf, fee, mocker, testdatadir) -> None
 
 def test_backtest_trim_no_data_left(default_conf, fee, mocker, testdatadir) -> None:
     default_conf['use_exit_signal'] = False
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
+    default_conf['max_open_trades'] = 10
+
+    mocker.patch(f'{EXMS}.get_fee', fee)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=float('inf'))
     patch_exchange(mocker)
     backtesting = Backtesting(default_conf)
     backtesting._set_strategy(backtesting.strategylist[0])
@@ -927,7 +1086,6 @@ def test_backtest_trim_no_data_left(default_conf, fee, mocker, testdatadir) -> N
         processed=deepcopy(processed),
         start_date=min_date,
         end_date=max_date,
-        max_open_trades=10,
     )
 
 
@@ -948,9 +1106,10 @@ def test_processed(default_conf, mocker, testdatadir) -> None:
 
 def test_backtest_dataprovider_analyzed_df(default_conf, fee, mocker, testdatadir) -> None:
     default_conf['use_exit_signal'] = False
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=100000)
+    default_conf['max_open_trades'] = 10
+    mocker.patch(f'{EXMS}.get_fee', fee)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=100000)
     patch_exchange(mocker)
     backtesting = Backtesting(default_conf)
     backtesting._set_strategy(backtesting.strategylist[0])
@@ -981,7 +1140,6 @@ def test_backtest_dataprovider_analyzed_df(default_conf, fee, mocker, testdatadi
         processed=deepcopy(processed),
         start_date=min_date,
         end_date=max_date,
-        max_open_trades=10,
     )
     assert count == 5
 
@@ -998,9 +1156,10 @@ def test_backtest_pricecontours_protections(default_conf, fee, mocker, testdatad
 
     default_conf['enable_protections'] = True
     default_conf['timeframe'] = '1m'
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
+    default_conf['max_open_trades'] = 1
+    mocker.patch(f'{EXMS}.get_fee', fee)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=float('inf'))
     tests = [
         ['sine', 9],
         ['raise', 10],
@@ -1024,7 +1183,6 @@ def test_backtest_pricecontours_protections(default_conf, fee, mocker, testdatad
             processed=processed,
             start_date=min_date,
             end_date=max_date,
-            max_open_trades=1,
         )
         assert len(results['results']) == numres
 
@@ -1047,9 +1205,9 @@ def test_backtest_pricecontours(default_conf, fee, mocker, testdatadir,
         default_conf['protections'] = protections
         default_conf['enable_protections'] = True
 
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=float('inf'))
+    mocker.patch(f'{EXMS}.get_fee', fee)
     # While entry-signals are unrealistic, running backtesting
     # over and over again should not cause different results
 
@@ -1062,11 +1220,12 @@ def test_backtest_pricecontours(default_conf, fee, mocker, testdatadir,
     processed = backtesting.strategy.advise_all_indicators(data)
     min_date, max_date = get_timerange(processed)
     assert isinstance(processed, dict)
+    backtesting.strategy.max_open_trades = 1
+    backtesting.config.update({'max_open_trades': 1})
     results = backtesting.backtest(
         processed=processed,
         start_date=min_date,
         end_date=max_date,
-        max_open_trades=1,
     )
     assert len(results['results']) == expected
 
@@ -1077,7 +1236,7 @@ def test_backtest_clash_buy_sell(mocker, default_conf, testdatadir):
         buy_value = 1
         sell_value = 1
         return _trend(dataframe, buy_value, sell_value)
-
+    default_conf['max_open_trades'] = 10
     backtest_conf = _make_backtest_conf(mocker, conf=default_conf, datadir=testdatadir)
     backtesting = Backtesting(default_conf)
     backtesting._set_strategy(backtesting.strategylist[0])
@@ -1094,6 +1253,7 @@ def test_backtest_only_sell(mocker, default_conf, testdatadir):
         sell_value = 1
         return _trend(dataframe, buy_value, sell_value)
 
+    default_conf['max_open_trades'] = 10
     backtest_conf = _make_backtest_conf(mocker, conf=default_conf, datadir=testdatadir)
     backtesting = Backtesting(default_conf)
     backtesting._set_strategy(backtesting.strategylist[0])
@@ -1104,9 +1264,10 @@ def test_backtest_only_sell(mocker, default_conf, testdatadir):
 
 
 def test_backtest_alternate_buy_sell(default_conf, fee, mocker, testdatadir):
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=float('inf'))
+    mocker.patch(f'{EXMS}.get_fee', fee)
+    default_conf['max_open_trades'] = 10
     backtest_conf = _make_backtest_conf(mocker, conf=default_conf,
                                         pair='UNITTEST/BTC', datadir=testdatadir)
     default_conf['timeframe'] = '1m'
@@ -1151,9 +1312,9 @@ def test_backtest_multi_pair(default_conf, fee, mocker, tres, pair, testdatadir)
         dataframe['exit_short'] = 0
         return dataframe
 
-    mocker.patch("freqtrade.exchange.Exchange.get_min_pair_stake_amount", return_value=0.00001)
-    mocker.patch("freqtrade.exchange.Exchange.get_max_pair_stake_amount", return_value=float('inf'))
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
+    mocker.patch(f"{EXMS}.get_min_pair_stake_amount", return_value=0.00001)
+    mocker.patch(f"{EXMS}.get_max_pair_stake_amount", return_value=float('inf'))
+    mocker.patch(f'{EXMS}.get_fee', fee)
     patch_exchange(mocker)
 
     pairs = ['ADA/BTC', 'DASH/BTC', 'ETH/BTC', 'LTC/BTC', 'NXT/BTC']
@@ -1165,6 +1326,7 @@ def test_backtest_multi_pair(default_conf, fee, mocker, tres, pair, testdatadir)
     if tres > 0:
         data[pair] = data[pair][tres:].reset_index()
     default_conf['timeframe'] = '5m'
+    default_conf['max_open_trades'] = 3
 
     backtesting = Backtesting(default_conf)
     backtesting._set_strategy(backtesting.strategylist[0])
@@ -1173,11 +1335,11 @@ def test_backtest_multi_pair(default_conf, fee, mocker, tres, pair, testdatadir)
 
     processed = backtesting.strategy.advise_all_indicators(data)
     min_date, max_date = get_timerange(processed)
+
     backtest_conf = {
         'processed': deepcopy(processed),
         'start_date': min_date,
         'end_date': max_date,
-        'max_open_trades': 3,
     }
 
     results = backtesting.backtest(**backtest_conf)
@@ -1195,11 +1357,12 @@ def test_backtest_multi_pair(default_conf, fee, mocker, tres, pair, testdatadir)
         backtesting.dataprovider.get_analyzed_dataframe('NXT/BTC', '5m')[0]
     ) == len(data['NXT/BTC']) - 1 - backtesting.strategy.startup_candle_count
 
+    backtesting.strategy.max_open_trades = 1
+    backtesting.config.update({'max_open_trades': 1})
     backtest_conf = {
         'processed': deepcopy(processed),
         'start_date': min_date,
         'end_date': max_date,
-        'max_open_trades': 1,
     }
     results = backtesting.backtest(**backtest_conf)
     assert len(evaluate_result_multi(results['results'], '5m', 1)) == 0
@@ -1460,7 +1623,7 @@ def test_backtest_start_futures_noliq(default_conf_usdt, mocker,
     patch_exchange(mocker)
 
     mocker.patch('freqtrade.plugins.pairlistmanager.PairListManager.whitelist',
-                 PropertyMock(return_value=['HULUMULU/USDT', 'XRP/USDT']))
+                 PropertyMock(return_value=['HULUMULU/USDT', 'XRP/USDT:USDT']))
     # mocker.patch('freqtrade.optimize.backtesting.Backtesting.backtest', backtestmock)
 
     patched_configuration_load_config_file(mocker, default_conf_usdt)
@@ -1491,7 +1654,7 @@ def test_backtest_start_nomock_futures(default_conf_usdt, mocker,
         "strategy": CURRENT_TEST_STRATEGY,
     })
     patch_exchange(mocker)
-    result1 = pd.DataFrame({'pair': ['XRP/USDT', 'XRP/USDT'],
+    result1 = pd.DataFrame({'pair': ['XRP/USDT:USDT', 'XRP/USDT:USDT'],
                             'profit_ratio': [0.0, 0.0],
                             'profit_abs': [0.0, 0.0],
                             'open_date': pd.to_datetime(['2021-11-18 18:00:00',
@@ -1507,7 +1670,7 @@ def test_backtest_start_nomock_futures(default_conf_usdt, mocker,
                             'close_rate': [0.104969, 0.103541],
                             'exit_reason': [ExitType.ROI, ExitType.ROI]
                             })
-    result2 = pd.DataFrame({'pair': ['XRP/USDT', 'XRP/USDT', 'XRP/USDT'],
+    result2 = pd.DataFrame({'pair': ['XRP/USDT:USDT', 'XRP/USDT:USDT', 'XRP/USDT:USDT'],
                             'profit_ratio': [0.03, 0.01, 0.1],
                             'profit_abs': [0.01, 0.02, 0.2],
                             'open_date': pd.to_datetime(['2021-11-19 18:00:00',
@@ -1552,7 +1715,7 @@ def test_backtest_start_nomock_futures(default_conf_usdt, mocker,
         }
     ])
     mocker.patch('freqtrade.plugins.pairlistmanager.PairListManager.whitelist',
-                 PropertyMock(return_value=['XRP/USDT']))
+                 PropertyMock(return_value=['XRP/USDT:USDT']))
     mocker.patch('freqtrade.optimize.backtesting.Backtesting.backtest', backtestmock)
 
     patched_configuration_load_config_file(mocker, default_conf_usdt)
@@ -1575,8 +1738,8 @@ def test_backtest_start_nomock_futures(default_conf_usdt, mocker,
         'up to 2021-11-21 04:00:00 (4 days).',
         'Backtesting with data from 2021-11-17 21:00:00 '
         'up to 2021-11-21 04:00:00 (3 days).',
-        'XRP/USDT, funding_rate, 8h, data starts at 2021-11-18 00:00:00',
-        'XRP/USDT, mark, 8h, data starts at 2021-11-18 00:00:00',
+        'XRP/USDT:USDT, funding_rate, 8h, data starts at 2021-11-18 00:00:00',
+        'XRP/USDT:USDT, mark, 8h, data starts at 2021-11-18 00:00:00',
         f'Running backtesting for Strategy {CURRENT_TEST_STRATEGY}',
     ]
 
