@@ -236,7 +236,7 @@ def test_store_backtest_candles(testdatadir, mocker):
 
     assert dump_mock.call_count == 1
     assert isinstance(dump_mock.call_args_list[0][0][0], Path)
-    assert str(dump_mock.call_args_list[0][0][0]).endswith(str('_signals.pkl'))
+    assert str(dump_mock.call_args_list[0][0][0]).endswith('_signals.pkl')
 
     dump_mock.reset_mock()
     # mock file exporting
@@ -245,7 +245,7 @@ def test_store_backtest_candles(testdatadir, mocker):
     assert dump_mock.call_count == 1
     assert isinstance(dump_mock.call_args_list[0][0][0], Path)
     # result will be testdatadir / testresult-<timestamp>_signals.pkl
-    assert str(dump_mock.call_args_list[0][0][0]).endswith(str('_signals.pkl'))
+    assert str(dump_mock.call_args_list[0][0][0]).endswith('_signals.pkl')
     dump_mock.reset_mock()
 
 
@@ -255,7 +255,7 @@ def test_write_read_backtest_candles(tmpdir):
 
     # test directory exporting
     stored_file = store_backtest_signal_candles(Path(tmpdir), candle_dict, '2022_01_01_15_05_13')
-    scp = open(stored_file, "rb")
+    scp = stored_file.open("rb")
     pickled_signal_candles = joblib.load(scp)
     scp.close()
 
@@ -269,7 +269,7 @@ def test_write_read_backtest_candles(tmpdir):
     # test file exporting
     filename = Path(tmpdir / 'testresult')
     stored_file = store_backtest_signal_candles(filename, candle_dict, '2022_01_01_15_05_13')
-    scp = open(stored_file, "rb")
+    scp = stored_file.open("rb")
     pickled_signal_candles = joblib.load(scp)
     scp.close()
 
