@@ -1,8 +1,7 @@
 """
 Unit test file for rpc/api_server.py
 """
-
-import json
+import asyncio
 import logging
 import time
 from datetime import datetime, timedelta, timezone
@@ -16,6 +15,7 @@ from fastapi import FastAPI, WebSocketDisconnect
 from fastapi.exceptions import HTTPException
 from fastapi.testclient import TestClient
 from requests.auth import _basic_auth_str
+from sqlalchemy import select
 
 from freqtrade.__init__ import __version__
 from freqtrade.enums import CandleType, RunMode, State, TradingMode
@@ -26,7 +26,7 @@ from freqtrade.rpc import RPC
 from freqtrade.rpc.api_server import ApiServer
 from freqtrade.rpc.api_server.api_auth import create_token, get_user_from_token
 from freqtrade.rpc.api_server.uvicorn_threaded import UvicornServer
-from tests.conftest import (CURRENT_TEST_STRATEGY, create_mock_trades, get_mock_coro,
+from tests.conftest import (CURRENT_TEST_STRATEGY, EXMS, create_mock_trades, get_mock_coro,
                             get_patched_freqtradebot, log_has, log_has_re, patch_get_signal)
 
 
@@ -68,22 +68,23 @@ def botclient(default_conf, mocker):
         ApiServer.shutdown()
 
 
-def client_post(client, url, data={}):
+def client_post(client: TestClient, url, data={}):
+
     return client.post(url,
-                       content=data,
+                       json=data,
                        headers={'Authorization': _basic_auth_str(_TEST_USER, _TEST_PASS),
                                 'Origin': 'http://example.com',
                                 'content-type': 'application/json'
                                 })
 
 
-def client_get(client, url):
+def client_get(client: TestClient, url):
     # Add fake Origin to ensure CORS kicks in
     return client.get(url, headers={'Authorization': _basic_auth_str(_TEST_USER, _TEST_PASS),
                                     'Origin': 'http://example.com'})
 
 
-def client_delete(client, url):
+def client_delete(client: TestClient, url):
     # Add fake Origin to ensure CORS kicks in
     return client.delete(url, headers={'Authorization': _basic_auth_str(_TEST_USER, _TEST_PASS),
                                        'Origin': 'http://example.com'})
@@ -299,10 +300,6 @@ def test_api_UvicornServer(mocker):
     s = UvicornServer(uvicorn.Config(MagicMock(), port=8080, host='127.0.0.1'))
     assert thread_mock.call_count == 0
 
-    s.install_signal_handlers()
-    # Original implementation starts a thread - make sure that's not the case
-    assert thread_mock.call_count == 0
-
     # Fake started to avoid sleeping forever
     s.started = True
     s.run_in_thread()
@@ -318,10 +315,6 @@ def test_api_UvicornServer_run(mocker):
     s = UvicornServer(uvicorn.Config(MagicMock(), port=8080, host='127.0.0.1'))
     assert serve_mock.call_count == 0
 
-    s.install_signal_handlers()
-    # Original implementation starts a thread - make sure that's not the case
-    assert serve_mock.call_count == 0
-
     # Fake started to avoid sleeping forever
     s.started = True
     s.run()
@@ -331,13 +324,10 @@ def test_api_UvicornServer_run(mocker):
 def test_api_UvicornServer_run_no_uvloop(mocker, import_fails):
     serve_mock = mocker.patch('freqtrade.rpc.api_server.uvicorn_threaded.UvicornServer.serve',
                               get_mock_coro(None))
+    asyncio.set_event_loop(asyncio.new_event_loop())
     s = UvicornServer(uvicorn.Config(MagicMock(), port=8080, host='127.0.0.1'))
     assert serve_mock.call_count == 0
 
-    s.install_signal_handlers()
-    # Original implementation starts a thread - make sure that's not the case
-    assert serve_mock.call_count == 0
-
     # Fake started to avoid sleeping forever
     s.started = True
     s.run()
@@ -474,9 +464,9 @@ def test_api_balance(botclient, mocker, rpc_balance, tickers):
     ftbot, client = botclient
 
     ftbot.config['dry_run'] = False
-    mocker.patch('freqtrade.exchange.Exchange.get_balances', return_value=rpc_balance)
-    mocker.patch('freqtrade.exchange.Exchange.get_tickers', tickers)
-    mocker.patch('freqtrade.exchange.Exchange.get_valid_pair_combination',
+    mocker.patch(f'{EXMS}.get_balances', return_value=rpc_balance)
+    mocker.patch(f'{EXMS}.get_tickers', tickers)
+    mocker.patch(f'{EXMS}.get_valid_pair_combination',
                  side_effect=lambda a, b: f"{a}/{b}")
     ftbot.wallets.update()
 
@@ -508,7 +498,7 @@ def test_api_count(botclient, mocker, ticker, fee, markets, is_short):
     ftbot, client = botclient
     patch_get_signal(ftbot)
     mocker.patch.multiple(
-        'freqtrade.exchange.Exchange',
+        EXMS,
         get_balances=MagicMock(return_value=ticker),
         fetch_ticker=ticker,
         get_fee=fee,
@@ -561,7 +551,7 @@ def test_api_locks(botclient):
     assert rc.json()['lock_count'] == 1
 
     rc = client_post(client, f"{BASE_URI}/locks/delete",
-                     data='{"pair": "XRP/BTC"}')
+                     data={"pair": "XRP/BTC"})
     assert_response(rc)
     assert rc.json()['lock_count'] == 0
 
@@ -595,7 +585,7 @@ def test_api_daily(botclient, mocker, ticker, fee, markets):
     ftbot, client = botclient
     patch_get_signal(ftbot)
     mocker.patch.multiple(
-        'freqtrade.exchange.Exchange',
+        EXMS,
         get_balances=MagicMock(return_value=ticker),
         fetch_ticker=ticker,
         get_fee=fee,
@@ -614,7 +604,7 @@ def test_api_trades(botclient, mocker, fee, markets, is_short):
     ftbot, client = botclient
     patch_get_signal(ftbot)
     mocker.patch.multiple(
-        'freqtrade.exchange.Exchange',
+        EXMS,
         markets=PropertyMock(return_value=markets)
     )
     rc = client_get(client, f"{BASE_URI}/trades")
@@ -625,7 +615,7 @@ def test_api_trades(botclient, mocker, fee, markets, is_short):
     assert rc.json()['offset'] == 0
 
     create_mock_trades(fee, is_short=is_short)
-    Trade.query.session.flush()
+    Trade.session.flush()
 
     rc = client_get(client, f"{BASE_URI}/trades")
     assert_response(rc)
@@ -645,7 +635,7 @@ def test_api_trade_single(botclient, mocker, fee, ticker, markets, is_short):
     ftbot, client = botclient
     patch_get_signal(ftbot, enter_long=not is_short, enter_short=is_short)
     mocker.patch.multiple(
-        'freqtrade.exchange.Exchange',
+        EXMS,
         markets=PropertyMock(return_value=markets),
         fetch_ticker=ticker,
     )
@@ -653,7 +643,7 @@ def test_api_trade_single(botclient, mocker, fee, ticker, markets, is_short):
     assert_response(rc, 404)
     assert rc.json()['detail'] == 'Trade not found.'
 
-    Trade.query.session.rollback()
+    Trade.rollback()
     create_mock_trades(fee, is_short=is_short)
 
     rc = client_get(client, f"{BASE_URI}/trade/3")
@@ -669,7 +659,7 @@ def test_api_delete_trade(botclient, mocker, fee, markets, is_short):
     stoploss_mock = MagicMock()
     cancel_mock = MagicMock()
     mocker.patch.multiple(
-        'freqtrade.exchange.Exchange',
+        EXMS,
         markets=PropertyMock(return_value=markets),
         cancel_order=cancel_mock,
         cancel_stoploss_order=stoploss_mock,
@@ -678,7 +668,7 @@ def test_api_delete_trade(botclient, mocker, fee, markets, is_short):
     create_mock_trades(fee, is_short=is_short)
 
     ftbot.strategy.order_types['stoploss_on_exchange'] = True
-    trades = Trade.query.all()
+    trades = Trade.session.scalars(select(Trade)).all()
     trades[1].stoploss_order_id = '1234'
     Trade.commit()
     assert len(trades) > 2
@@ -686,7 +676,7 @@ def test_api_delete_trade(botclient, mocker, fee, markets, is_short):
     rc = client_delete(client, f"{BASE_URI}/trades/1")
     assert_response(rc)
     assert rc.json()['result_msg'] == 'Deleted trade 1. Closed 1 open orders.'
-    assert len(trades) - 1 == len(Trade.query.all())
+    assert len(trades) - 1 == len(Trade.session.scalars(select(Trade)).all())
     assert cancel_mock.call_count == 1
 
     cancel_mock.reset_mock()
@@ -695,11 +685,11 @@ def test_api_delete_trade(botclient, mocker, fee, markets, is_short):
     assert_response(rc, 502)
     assert cancel_mock.call_count == 0
 
-    assert len(trades) - 1 == len(Trade.query.all())
+    assert len(trades) - 1 == len(Trade.session.scalars(select(Trade)).all())
     rc = client_delete(client, f"{BASE_URI}/trades/2")
     assert_response(rc)
     assert rc.json()['result_msg'] == 'Deleted trade 2. Closed 2 open orders.'
-    assert len(trades) - 2 == len(Trade.query.all())
+    assert len(trades) - 2 == len(Trade.session.scalars(select(Trade)).all())
     assert stoploss_mock.call_count == 1
 
     rc = client_delete(client, f"{BASE_URI}/trades/502")
@@ -707,6 +697,44 @@ def test_api_delete_trade(botclient, mocker, fee, markets, is_short):
     assert_response(rc, 502)
 
 
+@pytest.mark.parametrize('is_short', [True, False])
+def test_api_delete_open_order(botclient, mocker, fee, markets, ticker, is_short):
+    ftbot, client = botclient
+    patch_get_signal(ftbot, enter_long=not is_short, enter_short=is_short)
+    stoploss_mock = MagicMock()
+    cancel_mock = MagicMock()
+    mocker.patch.multiple(
+        EXMS,
+        markets=PropertyMock(return_value=markets),
+        fetch_ticker=ticker,
+        cancel_order=cancel_mock,
+        cancel_stoploss_order=stoploss_mock,
+    )
+
+    rc = client_delete(client, f"{BASE_URI}/trades/10/open-order")
+    assert_response(rc, 502)
+    assert 'Invalid trade_id.' in rc.json()['error']
+
+    create_mock_trades(fee, is_short=is_short)
+    Trade.commit()
+
+    rc = client_delete(client, f"{BASE_URI}/trades/5/open-order")
+    assert_response(rc, 502)
+    assert 'No open order for trade_id' in rc.json()['error']
+    trade = Trade.get_trades([Trade.id == 6]).first()
+    mocker.patch(f'{EXMS}.fetch_order', side_effect=ExchangeError)
+    rc = client_delete(client, f"{BASE_URI}/trades/6/open-order")
+    assert_response(rc, 502)
+    assert 'Order not found.' in rc.json()['error']
+
+    trade = Trade.get_trades([Trade.id == 6]).first()
+    mocker.patch(f'{EXMS}.fetch_order', return_value=trade.orders[-1].to_ccxt_object())
+
+    rc = client_delete(client, f"{BASE_URI}/trades/6/open-order")
+    assert_response(rc)
+    assert cancel_mock.call_count == 1
+
+
 def test_api_logs(botclient):
     ftbot, client = botclient
     rc = client_get(client, f"{BASE_URI}/logs")
@@ -743,7 +771,7 @@ def test_api_edge_disabled(botclient, mocker, ticker, fee, markets):
     ftbot, client = botclient
     patch_get_signal(ftbot)
     mocker.patch.multiple(
-        'freqtrade.exchange.Exchange',
+        EXMS,
         get_balances=MagicMock(return_value=ticker),
         fetch_ticker=ticker,
         get_fee=fee,
@@ -805,7 +833,7 @@ def test_api_profit(botclient, mocker, ticker, fee, markets, is_short, expected)
     ftbot, client = botclient
     patch_get_signal(ftbot)
     mocker.patch.multiple(
-        'freqtrade.exchange.Exchange',
+        EXMS,
         get_balances=MagicMock(return_value=ticker),
         fetch_ticker=ticker,
         get_fee=fee,
@@ -863,7 +891,7 @@ def test_api_stats(botclient, mocker, ticker, fee, markets, is_short):
     ftbot, client = botclient
     patch_get_signal(ftbot, enter_long=not is_short, enter_short=is_short)
     mocker.patch.multiple(
-        'freqtrade.exchange.Exchange',
+        EXMS,
         get_balances=MagicMock(return_value=ticker),
         fetch_ticker=ticker,
         get_fee=fee,
@@ -906,7 +934,7 @@ def test_api_performance(botclient, fee):
     )
     trade.close_profit = trade.calc_profit_ratio(trade.close_rate)
     trade.close_profit_abs = trade.calc_profit(trade.close_rate)
-    Trade.query.session.add(trade)
+    Trade.session.add(trade)
 
     trade = Trade(
         pair='XRP/ETH',
@@ -923,7 +951,7 @@ def test_api_performance(botclient, fee):
     trade.close_profit = trade.calc_profit_ratio(trade.close_rate)
     trade.close_profit_abs = trade.calc_profit(trade.close_rate)
 
-    Trade.query.session.add(trade)
+    Trade.session.add(trade)
     Trade.commit()
 
     rc = client_get(client, f"{BASE_URI}/performance")
@@ -944,7 +972,7 @@ def test_api_status(botclient, mocker, ticker, fee, markets, is_short,
     ftbot, client = botclient
     patch_get_signal(ftbot)
     mocker.patch.multiple(
-        'freqtrade.exchange.Exchange',
+        EXMS,
         get_balances=MagicMock(return_value=ticker),
         fetch_ticker=ticker,
         get_fee=fee,
@@ -969,13 +997,15 @@ def test_api_status(botclient, mocker, ticker, fee, markets, is_short,
         'close_profit_pct': None,
         'close_profit_abs': None,
         'close_rate': None,
-        'current_profit': ANY,
-        'current_profit_pct': ANY,
-        'current_profit_abs': ANY,
         'profit_ratio': ANY,
         'profit_pct': ANY,
         'profit_abs': ANY,
         'profit_fiat': ANY,
+        'total_profit_abs': ANY,
+        'total_profit_fiat': ANY,
+        'total_profit_ratio': ANY,
+        'realized_profit': 0.0,
+        'realized_profit_ratio': None,
         'current_rate': current_rate,
         'open_date': ANY,
         'open_timestamp': ANY,
@@ -1026,10 +1056,13 @@ def test_api_status(botclient, mocker, ticker, fee, markets, is_short,
         'liquidation_price': None,
         'funding_fees': None,
         'trading_mode': ANY,
+        'amount_precision': None,
+        'price_precision': None,
+        'precision_mode': None,
         'orders': [ANY],
     }
 
-    mocker.patch('freqtrade.exchange.Exchange.get_rate',
+    mocker.patch(f'{EXMS}.get_rate',
                  MagicMock(side_effect=ExchangeError("Pair 'ETH/BTC' not available")))
 
     rc = client_get(client, f"{BASE_URI}/status")
@@ -1062,7 +1095,7 @@ def test_api_blacklist(botclient, mocker):
 
     # Add ETH/BTC to blacklist
     rc = client_post(client, f"{BASE_URI}/blacklist",
-                     data='{"blacklist": ["ETH/BTC"]}')
+                     data={"blacklist": ["ETH/BTC"]})
     assert_response(rc)
     assert rc.json() == {"blacklist": ["DOGE/BTC", "HOT/BTC", "ETH/BTC"],
                          "blacklist_expanded": ["ETH/BTC"],
@@ -1072,7 +1105,7 @@ def test_api_blacklist(botclient, mocker):
                          }
 
     rc = client_post(client, f"{BASE_URI}/blacklist",
-                     data='{"blacklist": ["XRP/.*"]}')
+                     data={"blacklist": ["XRP/.*"]})
     assert_response(rc)
     assert rc.json() == {"blacklist": ["DOGE/BTC", "HOT/BTC", "ETH/BTC", "XRP/.*"],
                          "blacklist_expanded": ["ETH/BTC", "XRP/BTC", "XRP/USDT"],
@@ -1134,7 +1167,7 @@ def test_api_force_entry(botclient, mocker, fee, endpoint):
     ftbot, client = botclient
 
     rc = client_post(client, f"{BASE_URI}/{endpoint}",
-                     data='{"pair": "ETH/BTC"}')
+                     data={"pair": "ETH/BTC"})
     assert_response(rc, 502)
     assert rc.json() == {"error": f"Error querying /api/v1/{endpoint}: Force_entry not enabled."}
 
@@ -1142,9 +1175,9 @@ def test_api_force_entry(botclient, mocker, fee, endpoint):
     ftbot.config['force_entry_enable'] = True
 
     fbuy_mock = MagicMock(return_value=None)
-    mocker.patch("freqtrade.rpc.RPC._rpc_force_entry", fbuy_mock)
+    mocker.patch("freqtrade.rpc.rpc.RPC._rpc_force_entry", fbuy_mock)
     rc = client_post(client, f"{BASE_URI}/{endpoint}",
-                     data='{"pair": "ETH/BTC"}')
+                     data={"pair": "ETH/BTC"})
     assert_response(rc)
     assert rc.json() == {"status": "Error entering long trade for pair ETH/BTC."}
 
@@ -1168,10 +1201,10 @@ def test_api_force_entry(botclient, mocker, fee, endpoint):
         strategy=CURRENT_TEST_STRATEGY,
         trading_mode=TradingMode.SPOT
     ))
-    mocker.patch("freqtrade.rpc.RPC._rpc_force_entry", fbuy_mock)
+    mocker.patch("freqtrade.rpc.rpc.RPC._rpc_force_entry", fbuy_mock)
 
     rc = client_post(client, f"{BASE_URI}/{endpoint}",
-                     data='{"pair": "ETH/BTC"}')
+                     data={"pair": "ETH/BTC"})
     assert_response(rc)
     assert rc.json() == {
         'amount': 1.0,
@@ -1205,6 +1238,8 @@ def test_api_force_entry(botclient, mocker, fee, endpoint):
         'profit_pct': None,
         'profit_abs': None,
         'profit_fiat': None,
+        'realized_profit': 0.0,
+        'realized_profit_ratio': None,
         'fee_close': 0.0025,
         'fee_close_cost': None,
         'fee_close_currency': None,
@@ -1229,6 +1264,9 @@ def test_api_force_entry(botclient, mocker, fee, endpoint):
         'liquidation_price': None,
         'funding_fees': None,
         'trading_mode': 'spot',
+        'amount_precision': None,
+        'price_precision': None,
+        'precision_mode': None,
         'orders': [],
     }
 
@@ -1236,39 +1274,39 @@ def test_api_force_entry(botclient, mocker, fee, endpoint):
 def test_api_forceexit(botclient, mocker, ticker, fee, markets):
     ftbot, client = botclient
     mocker.patch.multiple(
-        'freqtrade.exchange.Exchange',
+        EXMS,
         get_balances=MagicMock(return_value=ticker),
         fetch_ticker=ticker,
         get_fee=fee,
         markets=PropertyMock(return_value=markets),
-        _is_dry_limit_order_filled=MagicMock(return_value=True),
+        _dry_is_price_crossed=MagicMock(return_value=True),
     )
     patch_get_signal(ftbot)
 
     rc = client_post(client, f"{BASE_URI}/forceexit",
-                     data='{"tradeid": "1"}')
+                     data={"tradeid": "1"})
     assert_response(rc, 502)
     assert rc.json() == {"error": "Error querying /api/v1/forceexit: invalid argument"}
-    Trade.query.session.rollback()
+    Trade.rollback()
 
     create_mock_trades(fee)
     trade = Trade.get_trades([Trade.id == 5]).first()
     assert pytest.approx(trade.amount) == 123
     rc = client_post(client, f"{BASE_URI}/forceexit",
-                     data='{"tradeid": "5", "ordertype": "market", "amount": 23}')
+                     data={"tradeid": "5", "ordertype": "market", "amount": 23})
     assert_response(rc)
     assert rc.json() == {'result': 'Created sell order for trade 5.'}
-    Trade.query.session.rollback()
+    Trade.rollback()
 
     trade = Trade.get_trades([Trade.id == 5]).first()
     assert pytest.approx(trade.amount) == 100
     assert trade.is_open is True
 
     rc = client_post(client, f"{BASE_URI}/forceexit",
-                     data='{"tradeid": "5"}')
+                     data={"tradeid": "5"})
     assert_response(rc)
     assert rc.json() == {'result': 'Created sell order for trade 5.'}
-    Trade.query.session.rollback()
+    Trade.rollback()
 
     trade = Trade.get_trades([Trade.id == 5]).first()
     assert trade.is_open is False
@@ -1418,7 +1456,7 @@ def test_api_pair_history(botclient, ohlcv_history):
                                   "No data for UNITTEST/BTC, 5m in 20200111-20200112 found.")
 
 
-def test_api_plot_config(botclient):
+def test_api_plot_config(botclient, mocker):
     ftbot, client = botclient
 
     rc = client_get(client, f"{BASE_URI}/plot_config")
@@ -1442,6 +1480,21 @@ def test_api_plot_config(botclient):
     assert isinstance(rc.json()['main_plot'], dict)
     assert isinstance(rc.json()['subplots'], dict)
 
+    rc = client_get(client, f"{BASE_URI}/plot_config?strategy=freqai_test_classifier")
+    assert_response(rc)
+    res = rc.json()
+    assert 'target_roi' in res['subplots']
+    assert 'do_predict' in res['subplots']
+
+    rc = client_get(client, f"{BASE_URI}/plot_config?strategy=HyperoptableStrategy")
+    assert_response(rc)
+    assert rc.json()['subplots'] == {}
+
+    mocker.patch('freqtrade.rpc.api_server.api_v1.get_rpc_optional', return_value=None)
+
+    rc = client_get(client, f"{BASE_URI}/plot_config")
+    assert_response(rc)
+
 
 def test_api_strategies(botclient, tmpdir):
     ftbot, client = botclient
@@ -1554,13 +1607,13 @@ def test_list_available_pairs(botclient):
         client, f"{BASE_URI}/available_pairs?timeframe=1h")
     assert_response(rc)
     assert rc.json()['length'] == 1
-    assert rc.json()['pairs'] == ['XRP/USDT']
+    assert rc.json()['pairs'] == ['XRP/USDT:USDT']
 
     rc = client_get(
         client, f"{BASE_URI}/available_pairs?timeframe=1h&candletype=mark")
     assert_response(rc)
     assert rc.json()['length'] == 2
-    assert rc.json()['pairs'] == ['UNITTEST/USDT', 'XRP/USDT']
+    assert rc.json()['pairs'] == ['UNITTEST/USDT:USDT', 'XRP/USDT:USDT']
     assert len(rc.json()['pair_interval']) == 2
 
 
@@ -1576,7 +1629,7 @@ def test_sysinfo(botclient):
 
 def test_api_backtesting(botclient, mocker, fee, caplog, tmpdir):
     ftbot, client = botclient
-    mocker.patch('freqtrade.exchange.Exchange.get_fee', fee)
+    mocker.patch(f'{EXMS}.get_fee', fee)
 
     rc = client_get(client, f"{BASE_URI}/backtest")
     # Backtest prevented in default mode
@@ -1616,7 +1669,7 @@ def test_api_backtesting(botclient, mocker, fee, caplog, tmpdir):
         "dry_run_wallet": 1000,
         "enable_protections": False
     }
-    rc = client_post(client, f"{BASE_URI}/backtest", data=json.dumps(data))
+    rc = client_post(client, f"{BASE_URI}/backtest", data=data)
     assert_response(rc)
     result = rc.json()
 
@@ -1667,7 +1720,7 @@ def test_api_backtesting(botclient, mocker, fee, caplog, tmpdir):
     assert result['status'] == 'running'
 
     # Post to backtest that's still running
-    rc = client_post(client, f"{BASE_URI}/backtest", data=json.dumps(data))
+    rc = client_post(client, f"{BASE_URI}/backtest", data=data)
     assert_response(rc, 502)
     result = rc.json()
     assert 'Bot Background task already running' in result['error']
@@ -1675,7 +1728,7 @@ def test_api_backtesting(botclient, mocker, fee, caplog, tmpdir):
     ApiServer._bgtask_running = False
 
     # Rerun backtest (should get previous result)
-    rc = client_post(client, f"{BASE_URI}/backtest", data=json.dumps(data))
+    rc = client_post(client, f"{BASE_URI}/backtest", data=data)
     assert_response(rc)
     result = rc.json()
     assert log_has_re('Reusing result of previous backtest.*', caplog)
@@ -1683,9 +1736,15 @@ def test_api_backtesting(botclient, mocker, fee, caplog, tmpdir):
     data['stake_amount'] = 101
 
     mocker.patch('freqtrade.optimize.backtesting.Backtesting.backtest_one_strategy',
-                 side_effect=DependencyException())
-    rc = client_post(client, f"{BASE_URI}/backtest", data=json.dumps(data))
-    assert log_has("Backtesting caused an error: ", caplog)
+                 side_effect=DependencyException('DeadBeef'))
+    rc = client_post(client, f"{BASE_URI}/backtest", data=data)
+    assert log_has("Backtesting caused an error: DeadBeef", caplog)
+
+    rc = client_get(client, f"{BASE_URI}/backtest")
+    assert_response(rc)
+    result = rc.json()
+    assert result['status'] == 'error'
+    assert 'Backtest failed' in result['status_msg']
 
     # Delete backtesting to avoid leakage since the backtest-object may stick around.
     rc = client_delete(client, f"{BASE_URI}/backtest")
@@ -1698,7 +1757,7 @@ def test_api_backtesting(botclient, mocker, fee, caplog, tmpdir):
 
     # Disallow base64 strategies
     data['strategy'] = "xx:cHJpbnQoImhlbGxvIHdvcmxkIik="
-    rc = client_post(client, f"{BASE_URI}/backtest", data=json.dumps(data))
+    rc = client_post(client, f"{BASE_URI}/backtest", data=data)
     assert_response(rc, 500)
 
 
@@ -1741,8 +1800,8 @@ def test_health(botclient):
 
     assert_response(rc)
     ret = rc.json()
-    assert ret['last_process_ts'] == 0
-    assert ret['last_process'] == '1970-01-01T00:00:00+00:00'
+    assert ret["last_process_ts"] is None
+    assert ret["last_process"] is None
 
 
 def test_api_ws_subscribe(botclient, mocker):
@@ -1766,7 +1825,7 @@ def test_api_ws_subscribe(botclient, mocker):
     assert sub_mock.call_count == 1
 
 
-def test_api_ws_requests(botclient, mocker, caplog):
+def test_api_ws_requests(botclient, caplog):
     caplog.set_level(logging.DEBUG)
 
     ftbot, client = botclient
